<?php

namespace App\Domain\Utils\Controllers;

use App\Domain\Utils\Requests\LoginRequest;
use Freelabois\LaravelQuickstart\Exceptions\BadCredentials;
use Freelabois\LaravelQuickstart\Exceptions\MissingCredentials;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\BadResponseException;
use Illuminate\Support\Facades\Lang;
use Psr\Http\Message\StreamInterface;

class PassportController
{

    /**
     * @param LoginRequest $request
     * @return StreamInterface|void
     * @throws BadCredentials
     * @throws MissingCredentials
     */
    public function login(LoginRequest $request)
    {
        $http = new Client();
        try {
            $response = $http->post(config('services.passport.login_endpoint'), [
                'form_params' => [
                    'grant_type' => 'password',
                    'client_id' => config('services.passport.client_id'),
                    'client_secret' => config('services.passport.client_secret'),
                    'username' => $request->username,
                    'password' => $request->password,
                ],
                'headers' => [
                    'EmpBanco' => $request->header('EmpBanco'),
                    'OrigemAplicacao' => $request->header('origin')
                ]
            ]);
            return $response->getBody();
        } catch (BadResponseException $e) {
            switch ($e->getCode()) {
                case 400:
                    throw new MissingCredentials(Lang::get('exceptions.login.invalid_request'), $e->getCode());
                case 401:
                    throw new BadCredentials(Lang::get('exceptions.login.incorrect_credentials'), $e->getCode());
                default:
                    throw $e;
            }
        }
        return;
    }

    public function logout()
    {
//        auth()->user()->tokens->each(function ($token) {
//            $token->delete();
//        });

        return response()->json(Lang::get('exceptions.login.logged_out_successfully'), 200);
    }

    public function authenticated()
    {
        return response()->json(['data' => 'authenticated']);
    }

}
