<?php

namespace App\Exceptions;

use Exception;
use Freelabois\LaravelQuickstart\Exceptions\BadRequest;
use Illuminate\Auth\AuthenticationException;
use Illuminate\Database\QueryException;
use Illuminate\Foundation\Exceptions\Handler as ExceptionHandler;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Lang;
use Illuminate\Validation\ValidationException;
use Throwable;
use Whoops\Handler\JsonResponseHandler;
use Whoops\Run;

class Handler extends ExceptionHandler
{
    /**
     * A list of the exception types that are not reported.
     *
     * @var array
     */
    protected $dontReport = [
        //
    ];

    /**
     * A list of the inputs that are never flashed for validation exceptions.
     *
     * @var array
     */
    protected $dontFlash = [
        'password',
        'password_confirmation',
    ];

    /**
     * Report or log an exception.
     *
     * @param Throwable $exception
     * @return void
     *
     * @throws Exception
     */
    public function report(Throwable $exception)
    {
        parent::report($exception);
    }

    /**
     * Render an exception into an HTTP response.
     *
     * @param Request $request
     * @param Throwable $exception
     * @return \Symfony\Component\HttpFoundation\Response
     *
     * @throws Throwable
     */
    public function render($request, Throwable $exception)
    {
        if (app()->environment('testing')) {
            throw $exception;
        }

        if ($exception instanceof AuthenticationException) {
            return response()->json(['error' => ['message' => Lang::get('exceptions.users.not_authenticated')]], 401);
        }

        if ($exception instanceof ValidationException) {
            $errors = $exception->errors();
            return response()->json([
                'error' => ['message' => $exception->getMessage(), 'errors' => $errors]
            ], 422);
        }

        if ($exception instanceof QueryException) {

            if($exception->getCode() == "23503"){
                return response()->json([
                    'error' => ['message' => "Não foi possível excluir. A entidade está associada a outras no sistema."]
                ], 409);
            }

            return response()->json([
                'error' => ['message' => $exception->getMessage()]
            ], 500);
        }

        if ($exception instanceof BadRequest) {
            return response()->json([
                'error' => ['message' => $exception->getMessage()],
            ], 400);
        }


        return self::renderJson($exception);
    }

    /**
     * Render an exception into a JSON HTTP response.
     *
     * @param Throwable $e
     * @return Response
     */
    public static function renderJson(Throwable $e)
    {
        $whoops = new Run;
        $whoops->pushHandler(new JsonResponseHandler);
        $whoops->sendHttpCode($e->getCode() ? $e->getCode() : 500);

        return new Response(
            $whoops->handleException($e)
        );
    }
}
